#!/usr/bin/env python3

# Copyright (C) 2020, Weizhi Song, Torsten Thomas.
# songwz03@gmail.com or t.thomas@unsw.edu.au

# MarkerMAG is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# MarkerMAG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import argparse
from Bio import SeqIO


uclust_16S_usage = '''
========================== uclust_16S example commands ==========================

module load usearch/10.0.240
MarkerMAG uclust_16S -in combined_16S.fasta -i 1 -out dereplicated_16S.fasta

=================================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def parse_uclust_output(seq_file_in, uclust_output_table, seq_file_out, cluster_to_member_file):

    seq_len_dict = {}
    for seq_record in SeqIO.parse(seq_file_in, 'fasta'):
        seq_len_dict[seq_record.id] = len(seq_record.seq)

    cluster_id_set = set()
    cluster_to_rep_seq_dict = {}
    cluster_to_seq_member_dict = {}
    for each_line in open(uclust_output_table):
        each_line_split = each_line.strip().split('\t')
        cluster_id = each_line_split[1]
        seq_id = each_line_split[8].split(' ')[0]
        cluster_id_set.add(int(cluster_id))

        if cluster_id not in cluster_to_rep_seq_dict:
            cluster_to_rep_seq_dict[cluster_id] = seq_id
            cluster_to_seq_member_dict[cluster_id] = {seq_id}
        else:
            if seq_len_dict[seq_id] > seq_len_dict[cluster_to_rep_seq_dict[cluster_id]]:
                cluster_to_rep_seq_dict[cluster_id] = seq_id
            cluster_to_seq_member_dict[cluster_id].add(seq_id)

    # write out cluster sequence members
    cluster_to_member_file_handle = open(cluster_to_member_file, 'w')
    for each_cluster in sorted([i for i in cluster_id_set]):
        cluster_to_member_file_handle.write('Cluster_%s\t%s\n' % (each_cluster, ','.join(sorted([i for i in cluster_to_seq_member_dict[str(each_cluster)]]))))
    cluster_to_member_file_handle.close()

    # write out the longest sequence in each cluster
    rep_seq_id_set = set()
    for i in cluster_to_rep_seq_dict:
        rep_seq_id_set.add(cluster_to_rep_seq_dict[i])

    rep_seq_file_handle = open(seq_file_out, 'w')
    for each_seq in SeqIO.parse(seq_file_in, 'fasta'):
        if each_seq.id in rep_seq_id_set:
            rep_seq_file_handle.write('>%s\n' % each_seq.id)
            rep_seq_file_handle.write('%s\n' % str(each_seq.seq))
    rep_seq_file_handle.close()


def uclust_16S(args):

    fasta_in    = args['in']
    iden_cutoff = args['i']
    fasta_out   = args['out']

    # define tmp file name
    fasta_out_path, fasta_out_basename, fasta_out_extension = sep_path_basename_ext(fasta_out)
    uclust_output_table             = '%s/%s.uc'                     % (fasta_out_path, fasta_out_basename)
    default_centroids_to_be_ignored = '%s/%s_default_centroids.%s'   % (fasta_out_path, fasta_out_basename, fasta_out_extension)
    cluster_to_member_file          = '%s/%s.uc.reorganised.txt'     % (fasta_out_path, fasta_out_basename)

    # run cluster_fast
    uclust_cmd = 'usearch -cluster_fast %s -id %s -query_cov 0.8 -target_cov 0.8 -centroids %s -uc %s -sort length -quiet' % (fasta_in, iden_cutoff, default_centroids_to_be_ignored, uclust_output_table)
    uclust_cmd = 'usearch -cluster_fast %s -id %s -centroids %s -uc %s -sort length -quiet' % (fasta_in, iden_cutoff, default_centroids_to_be_ignored, uclust_output_table)
    print(uclust_cmd)
    os.system(uclust_cmd)

    # extract the longest sequence from each cluster
    parse_uclust_output(fasta_in, uclust_output_table, fasta_out, cluster_to_member_file)

    # report
    print('Dereplicated sequences exported to: %s' % fasta_out)

if __name__ == '__main__':

    uclust_16S_parser = argparse.ArgumentParser()

    uclust_16S_parser.add_argument('-in', required=True, help='fasta file')
    uclust_16S_parser.add_argument('-i', required=False, type=float, default=1, help='Identity cutoff (0-1), default: 1')
    uclust_16S_parser.add_argument('-out', required=True, help='file out')
    args = vars(uclust_16S_parser.parse_args())

    uclust_16S(args)



import os
import glob
import shutil
import argparse
from Bio import SeqIO
import multiprocessing as mp


barrnap_16s_usage = '''
==================== barrnap_16s example commands ====================

module load perl/5.28.0
module load hmmer/3.2.1
module load bedtools/2.27.1
module load barrnap/0.9
MarkerMAG barrnap_16s -p Test -g genome_files -x fa -t 6 -force

======================================================================
'''

'''
module load python/3.7.3
source ~/mypython3env/bin/activate
module load perl/5.28.0
module load hmmer/3.2.1
module load bedtools/2.27.1
module load barrnap/0.9
cd /srv/scratch/z5039045/MarkerMAG_wd/MBARC26/link_mag_ref_wd_spade
python3 /srv/scratch/z5039045/MarkerMAG_wd/barrnap_16s.py -p SPAdes_refined -g Refined_refined_bins_renamed -x fna -t 12 -force
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


def barrnap_16s(args):

    output_prefix     = args['p']
    genome_folder     = args['g']
    genome_ext        = args['x']
    num_threads       = args['t']
    force_overwrite   = args['force']

    # define file name
    barrnap_16s_wd          = '%s_barrnap_16s_wd'               % output_prefix
    barrnap_op_foler        = '%s/%s_barrnap_outputs'           % (barrnap_16s_wd, output_prefix)
    output_seq_16s_folder   = '%s/%s_16S_seq'                   % (barrnap_16s_wd, output_prefix)
    combined_16s_seqs       = '%s/%s_16S.fasta'                 % (barrnap_16s_wd, output_prefix)
    output_table            = '%s/%s_16S.txt'                   % (barrnap_16s_wd, output_prefix)
    output_table_stats      = '%s/%s_16S_stats.txt'             % (barrnap_16s_wd, output_prefix)

    # create folder
    if (os.path.isdir(barrnap_16s_wd) is True) and (force_overwrite is False):
        print('Output folder detected, program exited: %s' % barrnap_16s_wd)
        exit()
    else:
        force_create_folder(barrnap_16s_wd)
        os.mkdir(barrnap_op_foler)

    genome_file_re = '%s/*.%s' % (genome_folder, genome_ext)
    genome_file_list = [os.path.basename(file_name) for file_name in glob.glob(genome_file_re)]
    genome_file_list_no_extension = ['.'.join(i.split('.')[:-1]) for i in genome_file_list]

    if len(genome_file_list) == 0:
        print('No query genome detected, program exited!')
        exit()

    # prepare commands for running barrnap
    argument_list_for_barrnap = []
    for genome in genome_file_list_no_extension:
        pwd_genome = '%s/%s.%s'  % (genome_folder, genome, genome_ext)
        pwd_op_ffn = '%s/%s.ffn' % (barrnap_op_foler, genome)
        pwd_op_gff = '%s/%s.gff' % (barrnap_op_foler, genome)
        pwd_op_log = '%s/%s.log' % (barrnap_op_foler, genome)
        barrnap_cmd = 'barrnap --quiet -o %s %s > %s 2> %s' % (pwd_op_ffn, pwd_genome, pwd_op_gff, pwd_op_log)
        argument_list_for_barrnap.append(barrnap_cmd)

    # run barrnap with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(os.system, argument_list_for_barrnap)
    pool.close()
    pool.join()

    # create folder
    os.mkdir(output_seq_16s_folder)

    # parse gff files
    genome_failed_barrnap = set()
    output_table_handle = open(output_table, 'w')
    output_table_handle.write('Genome\t16S\tLength(bp)\tLocation\tStart\tEnd\n')
    output_table_stats_handle = open(output_table_stats, 'w')
    output_table_stats_handle.write('Genome\tcopy_number\n')
    for genome in genome_file_list_no_extension:

        current_genome_gff          = '%s/%s.gff'       % (barrnap_op_foler, genome)
        current_genome_ffn          = '%s/%s.ffn'       % (barrnap_op_foler, genome)
        current_genome_16s_seq_file = '%s/%s_16S.ffn'   % (output_seq_16s_folder, genome)
        current_genome_fai_file     = '%s/%s.%s.fai'    % (genome_folder, genome, genome_ext)

        current_genome_16s_id_dict = {}
        seq_index_num = 1
        if os.path.isfile(current_genome_gff):
            for each_line in open(current_genome_gff):
                if (not each_line.startswith('#')) and ('16S_rRNA' in each_line):
                    each_line_split = each_line.strip().split('\t')
                    ctg_id = each_line_split[0]
                    start_pos = int(each_line_split[3])
                    end_pos = int(each_line_split[4])
                    length = end_pos - start_pos + 1
                    renamed_id_16s = '%s_16S_%s' % (genome, seq_index_num)
                    output_table_handle.write('%s\t%s\t%s\t%s\t%s\t%s\n' % (genome, renamed_id_16s, length, ctg_id, start_pos, end_pos))
                    current_genome_16s_id_dict[str(end_pos)] = renamed_id_16s
                    seq_index_num += 1
        else:
            genome_failed_barrnap.add(genome)

        # extract only 16S sequences
        current_genome_16s_seq_file_handle = open(current_genome_16s_seq_file, 'w')
        for seq_record in SeqIO.parse(current_genome_ffn, 'fasta'):
            seq_id = seq_record.id
            if seq_id.startswith('16S_rRNA'):
                seq_end_pos = seq_id.split(':')[3].split('(')[0].split('-')[-1]
                seq_id_new = current_genome_16s_id_dict[seq_end_pos]
                current_genome_16s_seq_file_handle.write('>%s\n' % seq_id_new)
                current_genome_16s_seq_file_handle.write('%s\n' % seq_record.seq)
        current_genome_16s_seq_file_handle.close()

        # remove index files
        if os.path.isfile(current_genome_fai_file):
            os.system('rm %s' % current_genome_fai_file)

        # write out copy number
        if len(current_genome_16s_id_dict) > 0:
            output_table_stats_handle.write('%s\t%s\n' % (genome, len(current_genome_16s_id_dict)))

    output_table_handle.close()
    output_table_stats_handle.close()

    # combine 16S sequences
    os.system('cat %s/*.ffn > %s' % (output_seq_16s_folder, combined_16s_seqs))
    os.system('rm -r %s' % output_seq_16s_folder)

    # report failed genomes
    if len(genome_failed_barrnap) > 0:
        print('Failed to run barrnap on the following genomes: %s' % ','.join(genome_failed_barrnap))

    print('Output table exported to %s' % output_table)
    print('Done!')


if __name__ == '__main__':

    barrnap_16s_parser = argparse.ArgumentParser(description='', usage=barrnap_16s_usage)
    barrnap_16s_parser.add_argument('-p',       required=True, type=str,               help='output prefix')
    barrnap_16s_parser.add_argument('-g',       required=True, type=str,               help='genome folder')
    barrnap_16s_parser.add_argument('-x',       required=False, default='fasta',       help='genome file extension, default: fasta')
    barrnap_16s_parser.add_argument('-t',       required=False, type=int, default=1,   help='number of threads, default: 1')
    barrnap_16s_parser.add_argument('-force',   required=False, action="store_true",   help='force overwrite existing results')

    args = vars(barrnap_16s_parser.parse_args())

    barrnap_16s(args)



#!/usr/bin/env python3

# Copyright (C) 2020, Weizhi Song, Torsten Thomas.
# songwz03@gmail.com or t.thomas@unsw.edu.au

# MarkerMAG is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# MarkerMAG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import argparse
from Bio import SeqIO
import multiprocessing as mp


rename_reads_usage = '''
=================== rename_reads example commands ===================

MarkerMAG rename_reads -r1 R1.fasta -r2 R2.fasta -p Soil -t 2
MarkerMAG rename_reads -r1 R1.fastq -r2 R2.fastq -p Soil -fq -t 2

Note: The order of paired reads in the two files must be the same.

=====================================================================
'''


def sep_path_basename_ext(file_in):

    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def rename_reads_worker(arg_list):

    file_in         = arg_list[0]
    file_out        = arg_list[1]
    output_prefix   = arg_list[2]
    file_format     = arg_list[3]
    read_strand     = arg_list[4]

    file_out_handle = open(file_out, 'w')
    read_index = 1
    for read_record in SeqIO.parse(file_in, file_format):
        if file_format == 'fasta':
            file_out_handle.write('>%s_%s.%s\n' % (output_prefix, read_index, read_strand))
            file_out_handle.write('%s\n' % read_record.seq)
        if file_format == 'fastq':
            read_record.id = '%s_%s.%s' % (output_prefix, read_index, read_strand)
            read_record.description = ''
            SeqIO.write(read_record, file_out_handle, file_format)
        read_index += 1
    file_out_handle.close()


def rename_reads(args):

    reads_r1        = args['r1']
    reads_r2        = args['r2']
    output_prefix   = args['p']
    fastq_format    = args['fq']
    num_threads     = args['t']

    # define name of output files
    format_to_parse = 'fasta'
    output_file_r1 = '%s_R1.fasta' % output_prefix
    output_file_r2 = '%s_R2.fasta' % output_prefix
    if fastq_format is True:
        format_to_parse = 'fastq'
        output_file_r1 = '%s_R1.fastq' % output_prefix
        output_file_r2 = '%s_R2.fastq' % output_prefix

    if os.path.isfile(output_file_r1) is True:
        print('%s detected, please remove your existing file or specify a different prefix' % output_file_r1)
        exit()
    if os.path.isfile(output_file_r2) is True:
        print('%s detected, please remove your existing file or specify a different prefix' % output_file_r2)
        exit()

    pool = mp.Pool(processes=num_threads)
    pool.map(rename_reads_worker, [[reads_r1, output_file_r1, output_prefix, format_to_parse, 1], [reads_r2, output_file_r2, output_prefix, format_to_parse, 2]])
    pool.close()
    pool.join()


if __name__ == '__main__':

    rename_reads_parser = argparse.ArgumentParser(description='', usage=rename_reads_usage)
    rename_reads_parser.add_argument('-r1',  required=True, type=str,               help='forward reads, fasta format')
    rename_reads_parser.add_argument('-r2',  required=True, type=str,               help='reverse reads, fasta format')
    rename_reads_parser.add_argument('-p',   required=True, type=str,               help='prefix of output file and read id')
    rename_reads_parser.add_argument('-fq',  required=False, action="store_true",   help='rename reads with quality score')
    rename_reads_parser.add_argument('-t',   required=False, type=int, default=1,   help='number of threads, default: 1')
    args = vars(rename_reads_parser.parse_args())
    rename_reads(args)



import argparse


def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if (each_element.isalpha() is True) or (each_element == '='):
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


def get_cigar_stats(cigar_splitted):

    # aligned_len: M I X =
    # clipping_len: S
    # mismatch_len: X I D
    # mismatch_pct = mismatch_len / aligned_len
    # aligned_pct  = aligned_len  / (aligned_len + clipping_len)
    # clipping_pct = clipping_len / (aligned_len + clipping_len)

    aligned_len = 0
    clipping_len = 0
    mismatch_len = 0
    for each_part in cigar_splitted:
        each_part_len = int(each_part[:-1])
        each_part_cate = each_part[-1]

        # get aligned_len
        if each_part_cate in {'M', 'm', 'I', 'i', 'X', 'x', '='}:
            aligned_len += each_part_len

        # get clipping_len
        if each_part_cate in ['S', 's']:
            clipping_len += each_part_len

        # get mismatch_len
        if each_part_cate in {'I', 'i', 'X', 'x', 'D', 'd'}:
            mismatch_len += each_part_len

    aligned_pct  = float("{0:.2f}".format(aligned_len * 100 / (aligned_len + clipping_len)))
    clipping_pct = float("{0:.2f}".format(clipping_len * 100 / (aligned_len + clipping_len)))
    mismatch_pct = float("{0:.2f}".format(mismatch_len * 100 / (aligned_len)))

    return aligned_len, aligned_pct, clipping_len, clipping_pct, mismatch_pct


def check_both_ends_clipping(cigar_splitted):

    both_ends_clipping = False
    if len(cigar_splitted) >= 3:
        if (cigar_splitted[0][-1] in ['S', 's']) and (cigar_splitted[-1][-1] in ['S', 's']):
            both_ends_clipping = True

    return both_ends_clipping


def remove_high_mismatch(sam_in, aln_len_cutoff, mismatch_cutoff, sam_out):

    sam_out_handle = open(sam_out, 'w')
    ref_len_dict = {}
    for each_read in open(sam_in):
        each_read_split = each_read.strip().split('\t')
        if each_read.startswith('@'):
            sam_out_handle.write(each_read)

            marker_id = ''
            marker_len = 0
            for each_element in each_read_split:
                if each_element.startswith('SN:'):
                    marker_id = each_element[3:]
                if each_element.startswith('LN:'):
                    marker_len = int(each_element[3:])
            ref_len_dict[marker_id] = marker_len

        else:
            ref_id = each_read_split[2]
            ref_pos = int(each_read_split[3])
            cigar = each_read_split[5]
            if cigar == '*':
                sam_out_handle.write(each_read)
            else:
                cigar_splitted = cigar_splitter(cigar)
                both_ends_clp = check_both_ends_clipping(cigar_splitted)
                if both_ends_clp is False:
                    r1_aligned_len, r1_aligned_pct, r1_clipping_len, r1_clipping_pct, r1_mismatch_pct = get_cigar_stats(cigar_splitted)
                    if r1_mismatch_pct <= mismatch_cutoff:

                        if r1_aligned_len >= aln_len_cutoff:

                            # check if clp in middle
                            if ('S' not in cigar) and ('s' not in cigar):
                                sam_out_handle.write(each_read)
                            else:
                                clip_in_middle = True
                                if (cigar_splitted[0][-1] in ['S', 's']) and (ref_pos == 1):
                                    clip_in_middle = False
                                if (cigar_splitted[-1][-1] in ['S', 's']):
                                    if (ref_pos + r1_aligned_len - 1) == ref_len_dict[ref_id]:
                                        clip_in_middle = False

                                if clip_in_middle is False:
                                    sam_out_handle.write(each_read)
    sam_out_handle.close()


# sam_file                       = 'FP_report_best.sam'
# sam_file_filtered              = 'FP_report_best_mis0.sam'
# aln_len_cutoff                 = 70
# mismatch_cutoff                = 0
#

parser = argparse.ArgumentParser()
parser.add_argument('-in', required=True, help='input file')
parser.add_argument('-out', required=True, help='output file')
parser.add_argument('-mm', required=True, type=int, help='mismatch cutoff')
parser.add_argument('-aln', required=True, type=int, help='alignment length cutoff')
args = vars(parser.parse_args())

sam_file                       = args['in']
sam_file_filtered              = args['out']
mismatch_cutoff                = args['mm']
aln_len_cutoff                 = args['aln']

remove_high_mismatch(sam_file, aln_len_cutoff, mismatch_cutoff, sam_file_filtered)



import os
from Bio import SeqIO


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def overlap_between_list(list_1, list_2):

    overlap = False
    for list_1_element in list_1:
        if list_1_element in list_2:
            overlap = True

    return overlap


def gnm_level_asessment(linked_mag_dict_rd1):

    rd1_correct_num = 0
    rd1_wrong_gnm = set()
    rd1_unknown_gnm = set()
    rd1_ambiguous_gnm = set()
    for each_rd_1_mag in linked_mag_dict_rd1:
        mag_assess = linked_mag_dict_rd1[each_rd_1_mag]
        if len(mag_assess) == 1:
            if mag_assess == {'Correct'}:
                rd1_correct_num += 1
            elif mag_assess == {'Wrong'}:
                rd1_wrong_gnm.add(each_rd_1_mag)
            elif mag_assess == {'Unknown'}:
                rd1_unknown_gnm.add(each_rd_1_mag)
        elif len(mag_assess) == 2:
            if ('Correct' in mag_assess) and ('Unknown' in mag_assess):
                rd1_correct_num += 1
            if ('Correct' in mag_assess) and ('Wrong' in mag_assess):
                rd1_ambiguous_gnm.add(each_rd_1_mag)
            if ('Wrong' in mag_assess) and ('Unknown' in mag_assess):
                rd1_wrong_gnm.add(each_rd_1_mag)
        elif len(mag_assess) == 3:
            rd1_ambiguous_gnm.add(each_rd_1_mag)

    return rd1_correct_num, rd1_unknown_gnm, rd1_wrong_gnm, rd1_ambiguous_gnm


########################################################################################################################

wd = '/Users/songweizhi/Desktop/assess_linkages_Oral'

########## reference to cluster ##########

drep_ani_cutoff             = 97
drep_cdb_file               = '%s/file_in/Cdb_%s.csv'                                       % (wd, drep_ani_cutoff)


########## 16S to reference ##########

perform_blastn_16s_vs_refs  = False  # True or False
combined_GI_ref_16S         = '%s/file_in/combined_Oral_ref_16S.ffn'                     % wd
matam_16s_seqs              = '%s/file_in/CAMI_Oral_138_16S_0.999.polished.fa'           % wd
matam_16s_blastn            = '%s/file_in/CAMI_Oral_138_16S_0.999.polished_vs_ref.tab'   % wd
iden_cutoff_16s             = 99.5  # 99.3 (best), 99.5
aln_len_cutoff_16s          = 500
cov_q_cutoff_16s            = 70
total_query_mag_num         = 87

matam_len_cutoff_list       = [500, 900, 1000, 1200, 1500]

'''
module load blast+/2.10.1
cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI_Oral/oral_799_ref_genomes_barrnap_16s_wd
makeblastdb -in combined_Oral_ref_16S.ffn -dbtype nucl -parse_seqids -logfile /dev/null

cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI_Oral
blastn -query CAMI_Oral_138_16S_0.999.polished.fa -db oral_799_ref_genomes_barrnap_16s_wd/combined_Oral_ref_16S.ffn -out CAMI_Oral_138_16S_0.999.polished_vs_ref.tab -evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 12
blastn -query CAMI_Oral_138_16S_0.999.polished_min900.fa -db oral_799_ref_genomes_barrnap_16s_wd/combined_Oral_ref_16S.ffn -out CAMI_Oral_138_16S_0.999.polished_min900_vs_ref.tab -evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 12
blastn -query CAMI_Oral_138_16S_0.999.polished_min1000.fa -db oral_799_ref_genomes_barrnap_16s_wd/combined_Oral_ref_16S.ffn -out CAMI_Oral_138_16S_0.999.polished_min1000_vs_ref.tab -evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 12
blastn -query CAMI_Oral_138_16S_0.999.polished_min1100.fa -db oral_799_ref_genomes_barrnap_16s_wd/combined_Oral_ref_16S.ffn -out CAMI_Oral_138_16S_0.999.polished_min1100_vs_ref.tab -evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 12
blastn -query CAMI_Oral_138_16S_0.999.polished_min1200.fa -db oral_799_ref_genomes_barrnap_16s_wd/combined_Oral_ref_16S.ffn -out CAMI_Oral_138_16S_0.999.polished_min1200_vs_ref.tab -evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 12
blastn -query CAMI_Oral_138_16S_0.999.polished_min1300.fa -db oral_799_ref_genomes_barrnap_16s_wd/combined_Oral_ref_16S.ffn -out CAMI_Oral_138_16S_0.999.polished_min1300_vs_ref.tab -evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 12
'''


################################################# reference to cluster #################################################

cluster_to_ref_dict = {}
ref_to_cluster_dict = {}
for each_ref in open(drep_cdb_file):
    if not each_ref.startswith('genome,secondary_cluster'):
        each_ref_split = each_ref.strip().split(',')
        ref_file_name = each_ref_split[0]
        ref_file_name_no_ext = '.'.join(ref_file_name.split('.')[:-1])
        ref_cluster = 'C' + each_ref_split[1]
        ref_to_cluster_dict[ref_file_name_no_ext] = ref_cluster
        if ref_cluster not in cluster_to_ref_dict:
            cluster_to_ref_dict[ref_cluster] = [ref_file_name_no_ext]
        else:
            cluster_to_ref_dict[ref_cluster].append(ref_file_name_no_ext)


################################################### 16S to reference ###################################################

blast_parameters = '-evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 1'
blast_cmd = 'blastn -query %s -subject %s -out %s %s' % (matam_16s_seqs, combined_GI_ref_16S, matam_16s_blastn, blast_parameters)
if perform_blastn_16s_vs_refs is True:
    os.system(blast_cmd)


for matam_len_cutoff in matam_len_cutoff_list:

    # get matam_16s_to_cluster_dict
    matam_16s_to_cluster_dict = {}
    for match in open(matam_16s_blastn):
        match_split = match.strip().split('\t')
        query = match_split[0]
        subject = match_split[1]
        subject_gnm = subject.split('_16S_')[0]
        subject_cluster = ref_to_cluster_dict[subject_gnm]
        iden = float(match_split[2])
        aln_len = int(match_split[3])
        query_len = int(match_split[12])
        subject_len = int(match_split[13])
        coverage_q = float(aln_len) * 100 / float(query_len)

        if query_len >= matam_len_cutoff:
            if (iden >= iden_cutoff_16s) and (aln_len > aln_len_cutoff_16s) and (coverage_q >= cov_q_cutoff_16s):
                if query not in matam_16s_to_cluster_dict:
                    matam_16s_to_cluster_dict[query] = {subject_cluster}
                else:
                    matam_16s_to_cluster_dict[query].add(subject_cluster)


    cluster_to_matam_16s_dict = {}
    for matam_16s in matam_16s_to_cluster_dict:
        matched_clusters =  matam_16s_to_cluster_dict[matam_16s]
        for matched_cluster in matched_clusters:
            if matched_cluster not in cluster_to_matam_16s_dict:
                cluster_to_matam_16s_dict[matched_cluster] = {matam_16s}
            else:
                cluster_to_matam_16s_dict[matched_cluster].add(matam_16s)


    print('%s\t%s/%s' % (matam_len_cutoff, len(cluster_to_matam_16s_dict), len(cluster_to_ref_dict)))




#!/usr/bin/env python3

import os
import argparse


subsample_reads_usage = '''
================== subsample_reads example commands ==================

MarkerMAG subsample_reads -r1 R1.fasta -r2 R2.fasta -ratio 0.1
MarkerMAG subsample_reads -r1 R1.fasta -r2 R2.fasta -ratio 0.1,0.3,0.5
MarkerMAG subsample_reads -r1 R1.fastq -r2 R2.fastq -ratio 0.1,0.3,0.5

======================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def subsample_reads(args):


    r1_file         = args['r1']
    r2_file         = args['r2']
    subsample_ratio = args['ratio']
    usearch_exe     = args['usearch']

    r1_file_path, r1_file_basename, r1_file_extension = sep_path_basename_ext(r1_file)
    r2_file_path, r2_file_basename, r2_file_extension = sep_path_basename_ext(r2_file)

    subsample_step_list = [float(i) for i in subsample_ratio.split(',')]

    for subsample_step in subsample_step_list:

        r1_file_out = '%s_%s%s' % (r1_file_basename, subsample_step, r1_file_extension)
        r2_file_out = '%s_%s%s' % (r2_file_basename, subsample_step, r2_file_extension)

        sunsample_cmd = ''
        if r1_file_extension != r2_file_extension:
            print('Different reads formats detected, program exited!')
            exit()
        elif ('q' in r1_file_extension) and ('q' in r2_file_extension):
            sunsample_cmd = '%s -fastx_subsample %s -reverse %s -fastqout %s -output2 %s -sample_pct %s' % (usearch_exe, r1_file, r2_file, r1_file_out, r2_file_out, subsample_step*100)
            os.system(sunsample_cmd)
        else:
            sunsample_cmd_r1 = '%s -fastx_subsample %s -fastaout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, r1_file, r1_file_out, subsample_step * 100)
            sunsample_cmd_r2 = '%s -fastx_subsample %s -fastaout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, r2_file, r2_file_out, subsample_step * 100)
            os.system(sunsample_cmd_r1)
            os.system(sunsample_cmd_r2)


if __name__ == '__main__':

    subsample_reads_parser = argparse.ArgumentParser(description='', usage=subsample_reads_usage)

    subsample_reads_parser.add_argument('-r1',      required=True, type=str,            help='forward reads')
    subsample_reads_parser.add_argument('-r2',      required=True, type=str,            help='reverse reads')
    subsample_reads_parser.add_argument('-ratio',   required=True, type=str,            help='subsample ratio, 0-1')
    subsample_reads_parser.add_argument('-usearch', required=False, default='usearch',  help='path to usearch executable file, default: usearch')

    args = vars(subsample_reads_parser.parse_args())
    subsample_reads(args)



#
# def sep_paired_and_singleton_reads(fasta_in, fasta_out_r1, fasta_out_r2, fasta_out_singleton):
#     reads_pair_dict = {}
#     for read_record in SeqIO.parse(fasta_in, 'fasta'):
#         read_id_base = '.'.join(read_record.id.split('.')[:-1])
#         read_strand = read_record.id.split('.')[-1]
#         if read_id_base not in reads_pair_dict:
#             reads_pair_dict[read_id_base] = {read_strand}
#         else:
#             reads_pair_dict[read_id_base].add(read_strand)
#
#     read_list_paired = set()
#     read_list_singleton = set()
#     for read_base in reads_pair_dict:
#         if len(reads_pair_dict[read_base]) == 1:
#             read_list_singleton.add(read_base)
#         if len(reads_pair_dict[read_base]) == 2:
#             read_list_paired.add(read_base)
#
#     fasta_out_r1_handle = open(fasta_out_r1, 'w')
#     fasta_out_r2_handle = open(fasta_out_r2, 'w')
#     fasta_out_singleton_handle = open(fasta_out_singleton, 'w')
#
#     for read_record in SeqIO.parse(fasta_in, 'fasta'):
#
#         read_id_base = '.'.join(read_record.id.split('.')[:-1])
#         read_strand = read_record.id.split('.')[-1]
#
#         if read_id_base in read_list_singleton:
#             fasta_out_singleton_handle.write('>%s\n' % read_record.id)
#             fasta_out_singleton_handle.write('%s\n' % str(read_record.seq))
#
#         if read_id_base in read_list_paired:
#
#             if read_strand == '1':
#                 fasta_out_r1_handle.write('>%s\n' % read_record.id)
#                 fasta_out_r1_handle.write('%s\n' % str(read_record.seq))
#
#             if read_strand == '2':
#                 fasta_out_r2_handle.write('>%s\n' % read_record.id)
#                 fasta_out_r2_handle.write('%s\n' % str(read_record.seq))
#
#     fasta_out_r1_handle.close()
#     fasta_out_r2_handle.close()
#     fasta_out_singleton_handle.close()
#
#
# def get_read_num_and_length(reads_file, tmp_file_location, seqtk_exe):
#
#     reads_file_line_num = '%s/R1_line_num.txt'  % (tmp_file_location)
#     reads_file_sub1000  = '%s/R1_sub1000.fasta' % (tmp_file_location)
#
#     # get the number of paired reads
#     os.system('wc -l %s > %s' % (reads_file, reads_file_line_num))
#     paired_reads_num = int(int(open(reads_file_line_num).readline().strip().split(' ')[0]) / 2)
#     if reads_file[-1] in ['Q', 'q']:
#         paired_reads_num = int(int(open(reads_file_line_num).readline().strip().split(' ')[0])/4)
#
#     # subsample 1000 reads
#     os.system('%s sample -s100 %s 1000 > %s' % (seqtk_exe, reads_file, reads_file_sub1000))
#
#     read_len_list = []
#     for each_seq in open(reads_file_sub1000):
#         if each_seq[0] not in ['>', '@', '+']:
#             read_len_list.append(len(each_seq.strip()))
#
#     read_len_median = np.median(read_len_list)
#     read_len_max    = np.max(read_len_list)
#
#     os.system('rm %s' % reads_file_line_num)
#     os.system('rm %s' % reads_file_sub1000)
#
#     return paired_reads_num, read_len_median, read_len_max
#
#
# def remove_clp_in_middle(sam_in, sam_out):
#
#     sam_out_handle = open(sam_out, 'w')
#
#     marker_len_dict = {}
#     for each_read in open(sam_in):
#         each_read_split = each_read.strip().split('\t')
#         if each_read.startswith('@'):
#             sam_out_handle.write(each_read)
#
#             marker_id = ''
#             marker_len = 0
#             for each_element in each_read_split:
#                 if each_element.startswith('SN:'):
#                     marker_id = each_element[3:]
#                 if each_element.startswith('LN:'):
#                     marker_len = int(each_element[3:])
#             marker_len_dict[marker_id] = marker_len
#         else:
#             cigar = each_read_split[5]
#
#             # check if clp in the middle
#             if ('S' not in cigar) and ('s' not in cigar):
#                 sam_out_handle.write(each_read)
#             else:
#                 ref_id = each_read_split[2]
#                 ref_pos = int(each_read_split[3])
#                 cigar_splitted = cigar_splitter(cigar)
#                 r1_aligned_len, r1_aligned_pct, r1_clipping_len, r1_clipping_pct, r1_mismatch_pct = get_cigar_stats(cigar_splitted)
#
#                 clip_in_middle = True
#                 if (cigar_splitted[0][-1] in ['S', 's']) and (ref_pos == 1):
#                     clip_in_middle = False
#                 if (cigar_splitted[-1][-1] in ['S', 's']):
#                     if (ref_pos + r1_aligned_len - 1) == marker_len_dict[ref_id]:
#                         clip_in_middle = False
#
#                 if clip_in_middle is False:
#                     sam_out_handle.write(each_read)
#
#     sam_out_handle.close()
#
#
# def get_ctg_mean_depth_by_samtools_coverage(index_ref, ref_seq, reads_r1, reads_r2, reads_unpaired, subsample_rate, num_threads):
#
#     ref_seq_file_path, ref_seq_file_basename, ref_seq_file_extension = sep_path_basename_ext(ref_seq)
#
#     sam_file                                                = '%s/%s.sam'                                               % (ref_seq_file_path, ref_seq_file_basename)
#     sam_file_one_end_clp                                    = '%s/%s_one_end_clp.sam'                                   % (ref_seq_file_path, ref_seq_file_basename)
#     sam_file_one_end_clp_no_middle                          = '%s/%s_one_end_clp_no_middle.sam'                         % (ref_seq_file_path, ref_seq_file_basename)
#     sam_file_one_end_clp_no_middle_best_match               = '%s/%s_one_end_clp_no_middle_best_match.sam'              % (ref_seq_file_path, ref_seq_file_basename)
#     sam_file_one_end_clp_no_middle_best_match_low_mismatch  = '%s/%s_one_end_clp_no_middle_best_match_low_mismatch.sam' % (ref_seq_file_path, ref_seq_file_basename)
#     sam_file_sorted                                         = '%s/%s_sorted.sam'                                        % (ref_seq_file_path, ref_seq_file_basename)
#     sam_file_sorted_log                                         = '%s/%s_sorted.log'                                        % (ref_seq_file_path, ref_seq_file_basename)
#     coverage_file                                           = '%s/%s_cov.txt'                                           % (ref_seq_file_path, ref_seq_file_basename)
#
#     # build reference index
#     cmd_bowtie2_build   = 'bowtie2-build --quiet --threads %s -f %s %s/%s' % (num_threads, ref_seq, ref_seq_file_path, ref_seq_file_basename)
#     if index_ref is True:
#         os.system(cmd_bowtie2_build)
#
#     # mapping
#     if reads_unpaired == '':
#         cmd_bowtie2_mapping = 'bowtie2 -x %s/%s -U %s,%s -S %s -p %s --xeq --local --all --no-unal -N 1 -L 30 -f --quiet' % (ref_seq_file_path, ref_seq_file_basename, reads_r1, reads_r2, sam_file, num_threads)
#     else:
#         cmd_bowtie2_mapping = 'bowtie2 -x %s/%s -U %s,%s,%s -S %s -p %s --xeq --local --all --no-unal -N 1 -L 30 -f --quiet' % (ref_seq_file_path, ref_seq_file_basename, reads_r1, reads_r2, reads_unpaired, sam_file, num_threads)
#     os.system(cmd_bowtie2_mapping)
#
#     # filter mapping
#     remove_both_ends_clp(sam_file, sam_file_one_end_clp)
#     remove_clp_in_middle(sam_file_one_end_clp, sam_file_one_end_clp_no_middle)
#     keep_best_matches_in_sam_keep_short_M(sam_file_one_end_clp_no_middle, 35, sam_file_one_end_clp_no_middle_best_match)
#     remove_high_mismatch(sam_file_one_end_clp_no_middle_best_match, 2, sam_file_one_end_clp_no_middle_best_match_low_mismatch)
#
#     # sort mapping
#     cmd_samtools_sort = 'samtools sort %s -o %s 2> %s' % (sam_file_one_end_clp_no_middle_best_match_low_mismatch, sam_file_sorted, sam_file_sorted_log)
#     os.system(cmd_samtools_sort)
#
#     # get mean depth
#     cmd_samtools_coverage = 'samtools coverage --ff 4 %s -o %s' % (sam_file_sorted, coverage_file)
#     os.system(cmd_samtools_coverage)
#
#     # remove sam files
#     os.system('rm %s' % sam_file)
#     # os.system('rm %s' % sam_file_sorted)
#
#     # store mean depth into dict
#     mean_depth_dict_ctg = {}
#     ctg_len_dict = {}
#     for each_ctg_depth in open(coverage_file):
#         if not each_ctg_depth.startswith('#'):
#             ctg_depth_split = each_ctg_depth.strip().split('\t')
#             ctg_id = ctg_depth_split[0]
#             ctg_len = int(ctg_depth_split[2])
#             ctg_depth = float(ctg_depth_split[6]) * (1 / subsample_rate)
#             mean_depth_dict_ctg[ctg_id] = ctg_depth
#             ctg_len_dict[ctg_id] = ctg_len
#
#     return mean_depth_dict_ctg, ctg_len_dict
#
#
# def get_ctg_mean_depth_by_samtools_coverage_global(index_ref, ref_seq, reads_r1, reads_r2, reads_unpaired, subsample_rate, num_threads):
#
#     ref_seq_file_path, ref_seq_file_basename, ref_seq_file_extension = sep_path_basename_ext(ref_seq)
#
#     sam_file                          = '%s/%s.sam'                         % (ref_seq_file_path, ref_seq_file_basename)
#     sam_file_best_match               = '%s/%s_best_match.sam'              % (ref_seq_file_path, ref_seq_file_basename)
#     sam_file_best_match_low_mismatch  = '%s/%s_best_match_low_mismatch.sam' % (ref_seq_file_path, ref_seq_file_basename)
#     sam_file_sorted                   = '%s/%s_sorted.sam'                  % (ref_seq_file_path, ref_seq_file_basename)
#     sam_file_sorted_log                   = '%s/%s_sorted.log'                  % (ref_seq_file_path, ref_seq_file_basename)
#     coverage_file                     = '%s/%s_cov.txt'                     % (ref_seq_file_path, ref_seq_file_basename)
#
#     # build reference index
#     cmd_bowtie2_build   = 'bowtie2-build --quiet --threads %s -f %s %s/%s' % (num_threads, ref_seq, ref_seq_file_path, ref_seq_file_basename)
#     if index_ref is True:
#         os.system(cmd_bowtie2_build)
#
#     # mapping
#     cmd_bowtie2_mapping = 'bowtie2 -x %s/%s -U %s,%s -S %s -p %s --all --no-unal -N 1 -L 30 -f --quiet' % (ref_seq_file_path, ref_seq_file_basename, reads_r1, reads_r2, sam_file, num_threads)
#     os.system(cmd_bowtie2_mapping)
#
#     # filter mapping
#     keep_best_matches_in_sam_keep_short_M(sam_file, 35, sam_file_best_match)
#     remove_high_mismatch(sam_file_best_match, 2, sam_file_best_match_low_mismatch)
#
#     # sort mapping
#     cmd_samtools_sort = 'samtools sort %s -o %s 2> %s' % (sam_file_best_match_low_mismatch, sam_file_sorted, sam_file_sorted_log)
#     os.system(cmd_samtools_sort)
#
#     # get mean depth
#     cmd_samtools_coverage = 'samtools coverage --ff 4 %s -o %s' % (sam_file_sorted, coverage_file)
#     os.system(cmd_samtools_coverage)
#
#     # remove sam files
#     os.system('rm %s' % sam_file)
#     # os.system('rm %s' % sam_file_sorted)
#
#     # store mean depth into dict
#     mean_depth_dict_ctg = {}
#     ctg_len_dict = {}
#     for each_ctg_depth in open(coverage_file):
#         if not each_ctg_depth.startswith('#'):
#             ctg_depth_split = each_ctg_depth.strip().split('\t')
#             ctg_id = ctg_depth_split[0]
#             ctg_len = int(ctg_depth_split[2])
#             ctg_depth = float(ctg_depth_split[6]) * (1 / subsample_rate)
#             mean_depth_dict_ctg[ctg_id] = ctg_depth
#             ctg_len_dict[ctg_id] = ctg_len
#
#     return mean_depth_dict_ctg, ctg_len_dict
#
#
# reads_file_r1_subset = '%s/input_R1_subset.fa' % mag_folder_in_wd
# reads_file_r2_subset = '%s/input_R2_subset.fa' % mag_folder_in_wd
#
# mean_depth_dict_gnm = {}
# if min_16s_gnm_multiple > 0:
#
#     if depth_file_mag is None:
#
#         report_and_log(
#             ('Round 1: depth info will be considered (but not provided!) for linking MAGs and 16S rRNA genes'),
#             pwd_log_file, keep_quiet)
#         report_and_log((
#                            'Round 1: depth estimation is time consuming, if you already have them, specify with -depth_16s and -depth_mag'),
#                        pwd_log_file, keep_quiet)
#         report_and_log(('Round 1: calculating MAG depth now, be patient!'), pwd_log_file, keep_quiet)
#
#         # get the number of paired reads
#         input_r1_line_num_file = '%s/R1_line_num.txt' % (mag_folder_in_wd)
#         input_r2_line_num_file = '%s/R2_line_num.txt' % (mag_folder_in_wd)
#         os.system('wc -l %s > %s' % (reads_file_r1, input_r1_line_num_file))
#         os.system('wc -l %s > %s' % (reads_file_r2, input_r2_line_num_file))
#         paired_r1_num = int(int(open(input_r1_line_num_file).readline().strip().split(' ')[0]) / 2)
#         paired_r2_num = int(int(open(input_r2_line_num_file).readline().strip().split(' ')[0]) / 2)
#         if reads_file_r1[-1] in ['Q', 'q']:
#             paired_r1_num = int(int(open(input_r1_line_num_file).readline().strip().split(' ')[0]) / 2)
#             paired_r2_num = int(int(open(input_r2_line_num_file).readline().strip().split(' ')[0]) / 2)
#
#         if paired_r1_num != paired_r2_num:
#             print('Inconsistent number of reads found in r1 and r2, program exited!')
#             exit()
#
#         # get the number of reads paired to subset
#         to_extract_reads_num = round(paired_r1_num * subsample_rate_for_depth_estimation)
#
#         # remember to use the same random seed to keep pairing
#         subsample_r1_cmd = '%s sample -s100 %s %s > %s' % (
#         seqtk_exe, reads_file_r1, to_extract_reads_num, reads_file_r1_subset)
#         subsample_r2_cmd = '%s sample -s100 %s %s > %s' % (
#         seqtk_exe, reads_file_r2, to_extract_reads_num, reads_file_r2_subset)
#
#         # subsample with multiprocessing
#         pool = mp.Pool(processes=2)
#         pool.map(os.system, [subsample_r1_cmd, subsample_r2_cmd])
#         pool.close()
#         pool.join()
#
#         # get mean depth for contig
#         mean_depth_dict_ctg, ctg_len_dict = get_ctg_mean_depth_by_samtools_coverage_global(True, combined_input_gnms,
#                                                                                            reads_file_r1_subset,
#                                                                                            reads_file_r2_subset, '',
#                                                                                            subsample_rate_for_depth_estimation,
#                                                                                            num_threads)
#
#         # write out ctg depth
#         depth_file_ctg_handle = open(depth_file_ctg, 'w')
#         for ctg in mean_depth_dict_ctg:
#             depth_file_ctg_handle.write('%s\t%s\n' % (ctg, mean_depth_dict_ctg[ctg]))
#         depth_file_ctg_handle.close()
#
#         # get mean_depth_dict_gnm
#         gnm_len_total_depth_dict = {}
#         for ctg in mean_depth_dict_ctg:
#             ctg_genome = ctg.split(gnm_to_ctg_connector)[0]
#             ctg_len = ctg_len_dict[ctg]
#             ctg_depth = mean_depth_dict_ctg[ctg]
#             ctg_total_depth = ctg_depth * ctg_len
#             if ctg_genome not in gnm_len_total_depth_dict:
#                 gnm_len_total_depth_dict[ctg_genome] = [ctg_len, ctg_total_depth]
#             else:
#                 gnm_len_total_depth_dict[ctg_genome][0] += ctg_len
#                 gnm_len_total_depth_dict[ctg_genome][1] += ctg_total_depth
#
#         for each_gnm in gnm_len_total_depth_dict:
#             gnm_len = gnm_len_total_depth_dict[each_gnm][0]
#             gnm_total_depth = gnm_len_total_depth_dict[each_gnm][1]
#             gnm_mean_depth = float("{0:.6f}".format(gnm_total_depth / gnm_len))
#             mean_depth_dict_gnm[each_gnm] = gnm_mean_depth
#
#         # write out gnm depth
#         depth_file_gnm_handle = open(depth_file_gnm, 'w')
#         for gnm in mean_depth_dict_gnm:
#             depth_file_gnm_handle.write('%s\t%s\n' % (gnm, mean_depth_dict_gnm[gnm]))
#         depth_file_gnm_handle.close()
#     else:
#         report_and_log(('Round 1: read in provided MAG depth'), pwd_log_file, keep_quiet)
#
#         # read in depth and store in mean_depth_dict_gnm here
#         for each_mag_depth in open(depth_file_mag):
#             each_mag_depth_split = each_mag_depth.strip().split('\t')
#             mag_id = each_mag_depth_split[0]
#             mag_depth = float(each_mag_depth_split[1])
#             mean_depth_dict_gnm[mag_id] = mag_depth
#



#!/usr/bin/env python3

import os
import glob
import shutil
import argparse
from Bio import SeqIO
from datetime import datetime
from distutils.spawn import find_executable


matam_16s_usage = '''
=================================== matam_16s example commands ===================================

MarkerMAG matam_16s -p Test -r1 R1.fastq -r2 R2.fastq -t 12 -d path/to/SILVA_138_SSURef_NR99
MarkerMAG matam_16s -p Test -r16s 16S_reads.fastq -t 12 -d path/to/SILVA_138_SSURef_NR99

# please refer to here for the preparation of input files
https://github.com/songweizhi/MarkerMAG/blob/master/doc/README_matam_16s.md

==================================================================================================
'''


def report_and_log(message_for_report, log_file, keep_quiet):

    time_format = '[%Y-%m-%d %H:%M:%S]'
    with open(log_file, 'a') as log_handle:
        log_handle.write('%s %s\n' % ((datetime.now().strftime(time_format)), message_for_report))

    if keep_quiet is False:
        print('%s %s' % ((datetime.now().strftime(time_format)), message_for_report))


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def str_to_num_list(nums_str):

    subsample_pct_list = []
    for pct_value in [str(float(i)) for i in nums_str.split(',')]:
        if pct_value[-2:] == '.0':
            subsample_pct_list.append(int(float(pct_value)))
        else:
            subsample_pct_list.append(float(pct_value))

    return sorted(subsample_pct_list)


def sep_paired_and_singleton_reads(seq_in, seq_out_r1, seq_out_r2, seq_out_singleton):

    input_seq_ext = os.path.splitext(seq_in)[1]
    input_seq_fmt = 'fasta'
    if ('q' in input_seq_ext) or ('Q' in input_seq_ext):
        input_seq_fmt = 'fastq'

    fmt_to_provide_for_write = 'fasta-2line'
    if input_seq_fmt == 'fastq':
        fmt_to_provide_for_write = 'fastq'

    reads_pair_dict = {}
    for read_record in SeqIO.parse(seq_in, input_seq_fmt):
        read_id_base = '.'.join(read_record.id.split('.')[:-1])
        read_strand = read_record.id.split('.')[-1]
        if read_id_base not in reads_pair_dict:
            reads_pair_dict[read_id_base] = {read_strand}
        else:
            reads_pair_dict[read_id_base].add(read_strand)

    read_list_paired = set()
    read_list_singleton = set()
    for read_base in reads_pair_dict:
        if len(reads_pair_dict[read_base]) == 1:
            read_list_singleton.add(read_base)
        if len(reads_pair_dict[read_base]) == 2:
            read_list_paired.add(read_base)

    fasta_out_r1_handle = open(seq_out_r1, 'w')
    fasta_out_r2_handle = open(seq_out_r2, 'w')
    fasta_out_singleton_handle = open(seq_out_singleton, 'w')
    for read_record in SeqIO.parse(seq_in, input_seq_fmt):
        read_id_base = '.'.join(read_record.id.split('.')[:-1])
        read_strand = read_record.id.split('.')[-1]
        if read_id_base in read_list_singleton:
            SeqIO.write(read_record, fasta_out_singleton_handle, fmt_to_provide_for_write)
        if read_id_base in read_list_paired:
            if read_strand == '1':
                SeqIO.write(read_record, fasta_out_r1_handle, fmt_to_provide_for_write)
            if read_strand == '2':
                SeqIO.write(read_record, fasta_out_r2_handle, fmt_to_provide_for_write)
    fasta_out_r1_handle.close()
    fasta_out_r2_handle.close()
    fasta_out_singleton_handle.close()


def single_line_seq(fasta_in, fasta_out):
    fasta_out_handle = open(fasta_out, 'w')
    for seq_record in SeqIO.parse(fasta_in, 'fasta'):
        fasta_out_handle.write('>%s\n' % seq_record.id)
        fasta_out_handle.write('%s\n' % seq_record.seq)
    fasta_out_handle.close()


def subsample_paired_and_singleton_reads(paired_r1, paired_r2, singleton_in, subsample_pct, paired_r1_out, paired_r2_out, singleton_out, usearch_exe):

    # check input file format
    input_seq_ext = os.path.splitext(paired_r1)[1]

    # define tmp files
    paired_r1_out_tmp = '%s.tmp' % paired_r1_out
    paired_r2_out_tmp = '%s.tmp' % paired_r2_out
    singleton_out_tmp = '%s.tmp' % singleton_out

    # prepare commands
    subsample_cmd_paired_r1 = '%s -fastx_subsample %s -fastaout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, paired_r1, paired_r1_out_tmp, subsample_pct)
    subsample_cmd_paired_r2 = '%s -fastx_subsample %s -fastaout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, paired_r2, paired_r2_out_tmp, subsample_pct)
    subsample_cmd_singleton = '%s -fastx_subsample %s -fastaout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, singleton_in, singleton_out_tmp, subsample_pct)
    if ('q' in input_seq_ext) or ('Q' in input_seq_ext):
        subsample_cmd_paired_r1 = '%s -fastx_subsample %s -fastqout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, paired_r1, paired_r1_out, subsample_pct)
        subsample_cmd_paired_r2 = '%s -fastx_subsample %s -fastqout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, paired_r2, paired_r2_out, subsample_pct)
        subsample_cmd_singleton = '%s -fastx_subsample %s -fastqout %s -sample_pct %s -randseed 1 -quiet' % (usearch_exe, singleton_in, singleton_out, subsample_pct)

    # execute commands
    os.system(subsample_cmd_paired_r1)
    os.system(subsample_cmd_paired_r2)
    os.system(subsample_cmd_singleton)

    if ('q' not in input_seq_ext) and ('Q' not in input_seq_ext):

        # single line sequences
        single_line_seq(paired_r1_out_tmp, paired_r1_out)
        single_line_seq(paired_r2_out_tmp, paired_r2_out)
        single_line_seq(singleton_out_tmp, singleton_out)

        # remove tmp files
        os.remove(paired_r1_out_tmp)
        os.remove(paired_r2_out_tmp)
        os.remove(singleton_out_tmp)


def subsample_sortmerna_output(sortmerna_op, subsample_pct, sortmerna_op_subsampled, usearch_exe, seqtk_exe):

    # define file name
    fasta_in_path,  fasta_in_basename,  fasta_in_ext  = sep_path_basename_ext(sortmerna_op)
    fasta_out_path, fasta_out_basename, fasta_out_ext = sep_path_basename_ext(sortmerna_op_subsampled)

    fasta_in_paired_r1               = '%s/%s_%s_paired_r1%s'            % (fasta_out_path, fasta_out_basename, fasta_in_basename, fasta_out_ext)
    fasta_in_paired_r2               = '%s/%s_%s_paired_r2%s'            % (fasta_out_path, fasta_out_basename, fasta_in_basename, fasta_out_ext)
    fasta_in_singleton               = '%s/%s_%s_singleton%s'            % (fasta_out_path, fasta_out_basename, fasta_in_basename, fasta_out_ext)
    fasta_in_paired_r1_subsampled    = '%s/%s_%s_paired_r1_subsampled%s' % (fasta_out_path, fasta_out_basename, fasta_in_basename, fasta_out_ext)
    fasta_in_paired_r2_subsampled    = '%s/%s_%s_paired_r2_subsampled%s' % (fasta_out_path, fasta_out_basename, fasta_in_basename, fasta_out_ext)
    fasta_in_singleton_subsampled    = '%s/%s_%s_singleton_subsampled%s' % (fasta_out_path, fasta_out_basename, fasta_in_basename, fasta_out_ext)
    fasta_in_paired_subsampled       = '%s/%s_%s_paired_subsampled%s'    % (fasta_out_path, fasta_out_basename, fasta_in_basename, fasta_out_ext)

    # separate paired and singleton reads
    sep_paired_and_singleton_reads(sortmerna_op, fasta_in_paired_r1, fasta_in_paired_r2, fasta_in_singleton)

    # subsample paired and singleton reads
    subsample_paired_and_singleton_reads(fasta_in_paired_r1, fasta_in_paired_r2, fasta_in_singleton,
                                         subsample_pct,
                                         fasta_in_paired_r1_subsampled, fasta_in_paired_r2_subsampled, fasta_in_singleton_subsampled,
                                         usearch_exe)

    # combine subsampled paired reads
    combine_paired_fasta_cmd = '%s mergepe %s %s > %s' % (seqtk_exe, fasta_in_paired_r1_subsampled, fasta_in_paired_r2_subsampled, fasta_in_paired_subsampled)
    os.system(combine_paired_fasta_cmd)

    # combine subsampled paired and singleton reads
    combine_all_subsample_reads_cmd = 'cat %s %s > %s' % (fasta_in_paired_subsampled, fasta_in_singleton_subsampled, sortmerna_op_subsampled)
    os.system(combine_all_subsample_reads_cmd)

    # remove tmp files
    os.remove(fasta_in_paired_r1)
    os.remove(fasta_in_paired_r2)
    os.remove(fasta_in_singleton)
    os.remove(fasta_in_paired_r1_subsampled)
    os.remove(fasta_in_paired_r2_subsampled)
    os.remove(fasta_in_singleton_subsampled)
    os.remove(fasta_in_paired_subsampled)


def prefix_seq(seq_in, prefix, seq_out):

    seq_out_handle = open(seq_out, 'w')
    for seq_record in SeqIO.parse(seq_in, 'fasta'):
        seq_id_new = '%s_%s' % (prefix, seq_record.id)
        seq_out_handle.write('>%s\n' % seq_id_new)
        seq_out_handle.write('%s\n' % str(seq_record.seq))
    seq_out_handle.close()


def parse_uclust_output(uclust_output_table, cluster_to_member_file):

    cluster_id_set = set()
    cluster_to_seq_member_dict = {}
    for each_line in open(uclust_output_table):
        each_line_split = each_line.strip().split('\t')
        cluster_id = each_line_split[1]
        seq_id = each_line_split[8].split(' ')[0]
        cluster_id_set.add(int(cluster_id))

        if cluster_id not in cluster_to_seq_member_dict:
            cluster_to_seq_member_dict[cluster_id] = {seq_id}
        else:
            cluster_to_seq_member_dict[cluster_id].add(seq_id)

    # write out cluster sequence members
    cluster_to_member_file_handle = open(cluster_to_member_file, 'w')
    for each_cluster in sorted([i for i in cluster_id_set]):
        cluster_to_member_file_handle.write('Cluster_%s\t%s\n' % (each_cluster, ','.join(sorted([i for i in cluster_to_seq_member_dict[str(each_cluster)]]))))
    cluster_to_member_file_handle.close()


def matam_16s(args):

    ###################################################### file in #####################################################

    # file in
    output_prefix                   = args['p']
    reads_file_r1                   = args['r1']
    reads_file_r2                   = args['r2']
    input_16S_reads                 = args['r16s']
    subsample_pcts                  = args['pct']
    matam_ref                       = args['d']
    uclust_iden_cutoff              = args['i']
    num_threads                     = args['t']
    mem_mb                          = args['mem']
    force_overwrite                 = args['force']
    matam_assembly_script           = args['matam']
    usearch_exe                     = args['usearch']
    seqtk_exe                       = args['seqtk']
    keep_quiet                      = args['quiet']


    ################################################ check dependencies ################################################

    program_list = [usearch_exe, seqtk_exe, matam_assembly_script]

    not_detected_programs = []
    for needed_program in program_list:
        if find_executable(needed_program) is None:
            not_detected_programs.append(needed_program)

    if not_detected_programs != []:
        print('%s not detected, program exited!' % ', '.join(not_detected_programs))
        exit()


    ####################################################################################################################

    extract_16s_reads = True
    if (reads_file_r1 is None) and (reads_file_r2 is None) and (input_16S_reads is not None):
        if os.path.isfile(input_16S_reads) is True:
            extract_16s_reads = False
        else:
            print('%s not found, program exited!' % os.path.basename(input_16S_reads))
            exit()
    elif (reads_file_r1 is not None) and (reads_file_r2 is not None) and (input_16S_reads is None):
        if os.path.isfile(reads_file_r1) is False:
            print('%s not found, program exited!' % os.path.basename(reads_file_r1))
            exit()
        if os.path.isfile(reads_file_r2) is False:
            print('%s not found, program exited!' % os.path.basename(reads_file_r2))
            exit()
    else:
        print('please either provide paired-end reads (-r1 and -r2) or 16S reads (-r16s) extracted from the paired-end reads as input, program exited!')
        exit()

    input_reads_ext = ''
    if extract_16s_reads is True:
        input_reads_ext = os.path.splitext(reads_file_r1)[1]
    else:
        input_reads_ext = os.path.splitext(input_16S_reads)[1]


    ####################################################################################################################

    matam16s_wd                             = '%s_Matam16S_wd'                          % output_prefix
    log_file                                = '%s/%s_matam_16s.log'                     % (matam16s_wd, output_prefix)
    combined_r1_r2                          = '%s/%s_combined_R1_R2%s'                  % (matam16s_wd, output_prefix, input_reads_ext)
    extracted_16S_reads                     = '%s/%s_16S_reads%s'                       % (matam16s_wd, output_prefix, input_reads_ext)
    combined_all_depth_matam_assemblies     = '%s/%s_assembled_16S_unclustered.fasta'   % (matam16s_wd, output_prefix)
    uclust_output_uc                        = '%s/%s_assembled_16S_uclust_%s.uc'        % (matam16s_wd, output_prefix, uclust_iden_cutoff)
    uclust_output_txt                       = '%s/%s_assembled_16S_uclust_%s.txt'       % (matam16s_wd, output_prefix, uclust_iden_cutoff)
    uclust_output_fasta                     = '%s/%s_assembled_16S_uclust_%s.fasta'     % (matam16s_wd, output_prefix, uclust_iden_cutoff)

    # create folder
    if os.path.isdir(matam16s_wd) is False:
        os.mkdir(matam16s_wd)
    else:
        if force_overwrite is False:
            print('Output folder detected, program exited: %s' % matam16s_wd)
            exit()
        else:
            os.system('rm -r %s' % matam16s_wd)
            os.mkdir(matam16s_wd)

    if extract_16s_reads is True:

        # combine R1 and R2
        report_and_log(('Combining R1 and R2 with seqtk'), log_file, keep_quiet)
        combined_r1_r2_cmd = '%s mergepe %s %s > %s' % (seqtk_exe, reads_file_r1, reads_file_r2, combined_r1_r2)
        report_and_log(combined_r1_r2_cmd, log_file, keep_quiet)
        os.system(combined_r1_r2_cmd)

        # Extract 16S reads
        report_and_log(('Extracting 16S reads with Matam'), log_file, keep_quiet)
        matam_filter_cmd = '%s -i %s -o %s/%s_get_16S_reads_wd --cpu %s --max_memory %s -v --filter_only -d %s' % (matam_assembly_script, combined_r1_r2, matam16s_wd, output_prefix, num_threads, mem_mb, matam_ref)
        report_and_log(matam_filter_cmd, log_file, keep_quiet)
        os.system(matam_filter_cmd)
        os.system('rm %s' % combined_r1_r2)

        # get file name of extracted 16S reads
        matam_16S_reads_re = '%s/%s_get_16S_reads_wd/workdir/*%s' % (matam16s_wd, output_prefix, input_reads_ext)
        matam_16S_reads_list = glob.glob(matam_16S_reads_re)
        if len(matam_16S_reads_list) == 0:
            print('16S reads not found, please check if 16S reads extraction step finished successfully.')
            exit()

        matam_16S_reads    = matam_16S_reads_list[0]
        os.system('mv %s %s' % (matam_16S_reads, extracted_16S_reads))
    else:
        extracted_16S_reads = input_16S_reads


    # subsample 16S reads and assemble
    renamed_matam_assembly_list = []
    for subsample_pct in str_to_num_list(subsample_pcts):

        report_and_log(('Subsample RNA reads at %s percent' % subsample_pct), log_file, keep_quiet)

        subsample_reads_file = '%s/%s_16S_reads_subset_%s%s'    % (matam16s_wd, output_prefix, subsample_pct, input_reads_ext)
        matam_output_folder  = '%s/%s_16S_reads_subset_%s_Matam_wd' % (matam16s_wd, output_prefix, subsample_pct)

        # subsample
        subsample_sortmerna_output(extracted_16S_reads, subsample_pct, subsample_reads_file, usearch_exe, seqtk_exe)

        # assemble with Matam
        report_and_log(('Assembling subsampled reads with Matam'), log_file, keep_quiet)

        matam_cmd = '%s -d %s -i %s --cpu %s --max_memory %s -v -o %s' % (matam_assembly_script, matam_ref, subsample_reads_file, num_threads, mem_mb, matam_output_folder)
        report_and_log(matam_cmd, log_file, keep_quiet)
        os.system(matam_cmd)

        matam_assemblies          = '%s/workdir/scaffolds.NR.min_500bp.fa'          % (matam_output_folder)
        matam_assemblies_prefixed = '%s/workdir/scaffolds.NR.min_500bp.prefixed.fa' % (matam_output_folder)
        seq_prefix                = '%s_subsample_%s'                               % (output_prefix, subsample_pct)

        if os.path.isfile(matam_assemblies) is True:
            report_and_log(('Adding prefix to Matam assemblies'), log_file, keep_quiet)
            prefix_seq(matam_assemblies, seq_prefix, matam_assemblies_prefixed)
            renamed_matam_assembly_list.append(matam_assemblies_prefixed)
        else:
            report_and_log(('No 16S rRNA gene sequence reconstructed at current depth!'), log_file, keep_quiet)

    # combine Matam outputs
    report_and_log(('Combine Matam assemblies at all depth'), log_file, keep_quiet)
    combine_cmd = 'cat %s > %s' % (' '.join(renamed_matam_assembly_list), combined_all_depth_matam_assemblies)
    report_and_log(combine_cmd, log_file, keep_quiet)
    os.system(combine_cmd)

    # dereplicate combined assemblies with Usearch
    report_and_log(('Cluster combined Matam assemblies with UCLUST at %s identity cutoff' % uclust_iden_cutoff), log_file, keep_quiet)
    uclust_cmd = '%s -cluster_fast %s -id %s -centroids %s -uc %s -sort length -quiet' % (usearch_exe, combined_all_depth_matam_assemblies, uclust_iden_cutoff, uclust_output_fasta, uclust_output_uc)
    report_and_log(uclust_cmd, log_file, keep_quiet)
    os.system(uclust_cmd)

    # get readable cluster results
    parse_uclust_output(uclust_output_uc, uclust_output_txt)

    # report
    report_and_log(('SortMeRNA identified 16S reads exported to %s' % os.path.basename(extracted_16S_reads)), log_file, keep_quiet)
    report_and_log(('Dereplicated Matam assemblies exported to %s'  % os.path.basename(uclust_output_fasta)), log_file, keep_quiet)
    report_and_log(('Done!'), log_file, keep_quiet)


if __name__ == '__main__':

    matam_16s_parser = argparse.ArgumentParser()
    matam_16s_parser.add_argument('-p',              required=True,  metavar='',                                            help='output prefix')
    matam_16s_parser.add_argument('-r1',             required=False, metavar='', default=None,                              help='forward reads')
    matam_16s_parser.add_argument('-r2',             required=False, metavar='', default=None,                              help='reverse reads')
    matam_16s_parser.add_argument('-r16s',           required=False, metavar='', default=None,                              help='extracted 16S reads, specify if you already have 16S reads identified from metagenomic dataset')
    matam_16s_parser.add_argument('-pct',            required=True,  metavar='', type=str, default='1,5,10,25,50,75,100',   help='subsample percentage, must be integer, between 1-100, deafault: 1,5,10,25,50,75,100')
    matam_16s_parser.add_argument('-d',              required=False, metavar='', type=str,                                  help='Matam ref db, controls "-d" in Matam')
    matam_16s_parser.add_argument('-i',              required=False, metavar='', type=float, default=0.999,                 help='cluster identity cutoff, do not provide value lower than 0.99, default: 0.999')
    matam_16s_parser.add_argument('-t',              required=False, metavar='', type=int, default=1,                       help='number of threads, default: 1')
    matam_16s_parser.add_argument('-mem',            required=False, metavar='', type=int, default=10240,                   help='maximum memory to be allocated to Matam (in MB), controls "--max_memory" in Matam, default: 10240')
    matam_16s_parser.add_argument('-force',          required=False, action="store_true",                                   help='force overwrite existing results')
    matam_16s_parser.add_argument('-quiet',          required=False, action="store_true",                                   help='not report progress')
    matam_16s_parser.add_argument('-matam',          required=False, metavar='', type=str, default='matam_assembly.py',     help='path to matam_assembly.py, default: matam_assembly.py')
    matam_16s_parser.add_argument('-seqtk',          required=False, metavar='', type=str, default='seqtk',                 help='path to seqtk executable file, default: seqtk')
    matam_16s_parser.add_argument('-usearch',        required=False, metavar='', type=str, default='usearch',               help='path to usearch executable file, default: usearch')
    args = vars(matam_16s_parser.parse_args())
    matam_16s(args)



#!/usr/bin/env python3

import os
import glob
import shutil
import argparse
from Bio import SeqIO
from datetime import datetime
from distutils.spawn import find_executable


polish_16s_usage = '''
===================== polish_16s example commands =====================

MarkerMAG polish_16s -in Matam_output.fa -out Matam_output_polished.fa

=======================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def polish_16s(args):

    file_in = args['in']
    file_out_ffn = args['out']

    file_out_path, file_out_base, file_out_ext = sep_path_basename_ext(file_out_ffn)

    barrnap_stdout   = '%s/%s.log'    % (file_out_path, file_out_base)
    file_out_gff     = '%s/%s.gff'    % (file_out_path, file_out_base)
    file_out_ffn_tmp = '%s/%s_tmp%s' % (file_out_path, file_out_base, file_out_ext)

    barrnap_cmd = 'barrnap --quiet -o %s %s 2> %s > %s' % (file_out_ffn_tmp, file_in, barrnap_stdout, file_out_gff)
    os.system(barrnap_cmd)

    wrote_id = []
    file_out_ffn_handle = open(file_out_ffn, 'w')
    for each_16s in SeqIO.parse(file_out_ffn_tmp, 'fasta'):
        seq_id = each_16s.id
        if seq_id.startswith('16S_rRNA::'):
            seq_id_polished = seq_id[10:].split(':')[0]

            if seq_id_polished not in wrote_id:
                file_out_ffn_handle.write('>%s\n' % seq_id_polished)
                file_out_ffn_handle.write('%s\n' % str(each_16s.seq))
                wrote_id.append(seq_id_polished)
            else:
                file_out_ffn_handle.write('>%s_%s\n' % (seq_id_polished, (wrote_id.count(seq_id_polished) + 1)))
                file_out_ffn_handle.write('%s\n' % str(each_16s.seq))
                wrote_id.append(seq_id_polished)

    file_out_ffn_handle.close()

    os.system('rm %s' % file_out_ffn_tmp)
    os.system('rm %s.fai' % file_in)


if __name__ == '__main__':

    matam_16s_parser = argparse.ArgumentParser()

    matam_16s_parser.add_argument('-in',  required=True, help='input 16S sequences')
    matam_16s_parser.add_argument('-out', required=True, help='output sequences')

    args = vars(matam_16s_parser.parse_args())
    polish_16s(args)



import os
from setuptools import setup


def version():

    setup_dir = os.path.dirname(os.path.realpath(__file__))
    version_file = open(os.path.join(setup_dir, 'MarkerMAG', 'VERSION'))

    return version_file.readline().strip()


__long_description__ = '''

MarkerMAG: linking MAGs with 16S rRNA marker genes

Weizhi Song, Torsten Thomas

Center for Marine Science & Innovation (CMSI)

University of New South Wales, Sydney, Australia

'''

setup(name="MarkerMAG",
      version=version(),
      long_description=__long_description__,
      license="GPL3+",
      author="Weizhi Song, Torsten Thomas",
      author_email="songwz03@gmail.com",
      keywords="Bioinformatics Metagenomics MAG 16S",
      description="linking MAGs with 16S rRNA marker genes",
      url="https://github.com/songweizhi/MarkerMAG",
      packages=['MarkerMAG', 'MarkerMAG.vxtractor'],
      package_data={'': ['*', '*/*', '*/*/*']},
      include_package_data=True,
      install_requires=['biopython', 'pandas', 'plotly', 'seaborn', 'matplotlib', 'numpy'],
      scripts=['bin/MarkerMAG'])



#
# # calculate depth for input 16S
# mean_depth_dict_16s = {}
# if min_16s_gnm_multiple > 0:
#
#     if depth_file_16s is None:
#
#         print('Round 1: calculating depth for 16S')
#         # get mean depth for 16S sequences
#         mean_depth_dict_16s, s16_len_dict = get_ctg_mean_depth_by_samtools_coverage_global(True, input_16s_qc,
#                                                                                            reads_file_r1_subset,
#                                                                                            reads_file_r2_subset, '',
#                                                                                            subsample_rate_for_depth_estimation,
#                                                                                            num_threads)
#
#         # write out 16s depth
#         depth_file_16s_handle = open(depth_file_16s_calculated, 'w')
#         for s16 in mean_depth_dict_16s:
#             depth_file_16s_handle.write('%s\t%s\n' % (s16, mean_depth_dict_16s[s16]))
#         depth_file_16s_handle.close()
#     else:
#         print('Round 1: read in provided 16S depth')
#         # read in depth and store in mean_depth_dict_gnm here
#         for each_16s_depth in open(depth_file_16s):
#             each_16s_depth_split = each_16s_depth.strip().split('\t')
#             s16_id = each_16s_depth_split[0]
#             s16_depth = float(each_16s_depth_split[1])
#             mean_depth_dict_16s[s16_id] = s16_depth